Attribute VB_Name = "Globals"
Option Explicit

' Folder browsing API declarations.
Private Type BrowseInfo
    hWndOwner As Long
    pidlRoot As Long
    sDisplayName As String
    sTitle As String
    ulFlags As Long
    lpfn As Long
    lParam As Long
    iImage As Long
End Type

Private Declare Function SHBrowseForFolder Lib "shell32.dll" (bBrowse As BrowseInfo) As Long
Private Declare Function SHGetPathFromIDList Lib "shell32.dll" (ByVal lItem As Long, ByVal sDir As String) As Long

' File listing API declarations.
Private Const MAX_PATH = 260
Private Type FILETIME
    dwLowDateTime As Long
    dwHighDateTime As Long
End Type
Private Type WIN32_FIND_DATA
    dwFileAttributes As Long
    ftCreationTime As FILETIME
    ftLastAccessTime As FILETIME
    ftLastWriteTime As FILETIME
    nFileSizeHigh As Long
    nFileSizeLow As Long
    dwReserved0 As Long
    dwReserved1 As Long
    cFileName As String * MAX_PATH
    cAlternate As String * 14
End Type

Private Declare Function FindFirstFile Lib "kernel32" Alias "FindFirstFileA" (ByVal lpFileName As String, lpFindFileData As WIN32_FIND_DATA) As Long
Private Declare Function FindNextFile Lib "kernel32" Alias "FindNextFileA" (ByVal hFindFile As Long, lpFindFileData As WIN32_FIND_DATA) As Long
Private Declare Function FindClose Lib "kernel32" (ByVal hFindFile As Long) As Long
Private Declare Function GetLastError Lib "kernel32" () As Long

Private Const ERROR_NO_MORE_FILES = 18&
Private Const INVALID_HANDLE_VALUE = -1
Private Const DDL_DIRECTORY = &H10

' Short file name API declarations.
Private Declare Function GetShortPathName Lib "kernel32" Alias "GetShortPathNameA" (ByVal lpszLongPath As String, ByVal lpszShortPath As String, ByVal cchBuffer As Long) As Long

' Return the short file name for a long file name.
Public Function ShortFileName(ByVal long_name As String) As String
Dim length As Long
Dim short_name As String

    short_name = Space$(1024)
    length = GetShortPathName( _
        long_name, short_name, _
        Len(short_name))
    If length < 1 Then
        MsgBox "Error converting path '" & _
            long_name & "' into a short name", _
            vbExclamation Or vbOKOnly, "Path Error"
    Else
        ShortFileName = Left$(short_name, length)
    End If
End Function

' Return the long file name for a short file name.
Public Function LongFileName(ByVal short_name As String) As String
Dim pos As Integer
Dim result As String
Dim long_name As String

    ' Start after the drive letter if any.
    If Mid$(short_name, 2, 1) = ":" Then
        result = Left$(short_name, 2)
        pos = 3
    Else
        result = ""
        pos = 1
    End If

    ' Consider each section in the file name.
    Do While pos > 0
        ' Find the next \.
        pos = InStr(pos + 1, short_name, "\")

        ' Get the next piece of the path.
        If pos = 0 Then
            long_name = Dir$(short_name, vbNormal + vbHidden + vbSystem + vbDirectory)
        Else
            long_name = Dir$(Left$(short_name, pos - 1), vbNormal + vbHidden + vbSystem + vbDirectory)
        End If
        result = result & "\" & long_name
    Loop

    LongFileName = result
End Function

' Randomize an array of integers indexed from 1.
Public Sub RandomizeArray(indexes() As Integer)
Dim num_items As Integer
Dim i As Integer
Dim j As Integer
Dim tmp As Integer

    ' Randomize the array.
    Randomize
    num_items = UBound(indexes)
    For i = 1 To num_items - 1
        ' Pick a random entry.
        j = Int((num_items - i + 1) * Rnd + i)

        ' Swap the numbers.
        tmp = indexes(i)
        indexes(i) = indexes(j)
        indexes(j) = tmp
    Next i
End Sub

' List all music files in the directory.
Public Sub ListFiles(ByVal start_dir As String, ByVal directories As Collection, ByVal file_names As Collection)
Dim fname As String
Dim search_handle As Long
Dim file_data As WIN32_FIND_DATA

    start_dir = ShortFileName(start_dir)

    ' Get the first file.
    search_handle = FindFirstFile(start_dir & "\*.mp3", file_data)
    If search_handle <> INVALID_HANDLE_VALUE Then
        ' Get the rest of the files.
        Do
            fname = file_data.cFileName
            fname = Left$(fname, InStr(fname, Chr$(0)) - 1)
            file_names.Add fname
            directories.Add start_dir

            ' Get the next file.
            If FindNextFile(search_handle, file_data) = 0 Then Exit Do
        Loop

        ' Close the file search hanlde.
        FindClose search_handle
    End If
End Sub

' Let the user browse for a directory. Return the
' selected directory. Return an empty string if
' the user cancels.
Public Function BrowseForDirectory(ByVal hwnd As Long) As String
Dim browse_info As BrowseInfo
Dim item As Long
Dim dir_name As String
   
   browse_info.hWndOwner = hwnd
   browse_info.pidlRoot = 0
   browse_info.sDisplayName = Space$(260)
   browse_info.sTitle = "Select Directory"
   browse_info.ulFlags = 1 ' Return directory name.
   browse_info.lpfn = 0
   browse_info.lParam = 0
   browse_info.iImage = 0
   
   item = SHBrowseForFolder(browse_info)
   If item Then
       dir_name = Space$(260)
       If SHGetPathFromIDList(item, dir_name) Then
           BrowseForDirectory = Left(dir_name, InStr(dir_name, Chr$(0)) - 1)
       Else
           BrowseForDirectory = ""
       End If
   End If
End Function

